/*----------------------------------------------------------------------
  File Name          : XL372.h
  Author             : MPD Application Team
  Version            : V1.0
  Date               : 12/03/2016
  Description        : XL372 Registers and bits values
  File ID            : $Id: XL372.h,v 1.1.1.1 2011/12/02$

  Analog Devices ADXL 372 digital output accellerometer 
  with advanced digital features.

  Based on preliminary data sheet Rev 12b

  (c) 2011 Analog Devices application support team.
  xxx@analog.com

  ----------------------------------------------------------------------

  The present firmware which is for guidance only aims at providing
  customers with coding information regarding their products in order
  for them to save time.  As a result, Analog Devices shall not be
  held liable for any direct, indirect or consequential damages with
  respect to any claims arising from the content of such firmware and/or
  the use made by customers of the coding information contained herein
  in connection with their products.

----------------------------------------------------------------------*/

#ifndef __XL372_H
#define __XL372_H

/* ------- Register names ------- */
#define XL372_DEVID_AD                 0x00
#define XL372_DEVID_MST                0x01
#define XL372_PARTID                   0x02
#define XL372_REVID                    0x03
#define XL372_STATUS                   0x04
#define XL372_STATUS2                  0x05
#define XL372_FIFO_ENTRIES2            0x06
#define XL372_FIFO_ENTRIES             0x07

#define XL372_XDATA_H                  0x08
#define XL372_XDATA_L                  0x09

#define XL372_YDATA_H                  0x0A
#define XL372_YDATA_L                  0x0B

#define XL372_ZDATA_H                  0x0C
#define XL372_ZDATA_L                  0x0D

#define XL372_MAXPEAK_X_H              0x15
#define XL372_MAXPEAK_X_L              0x16

#define XL372_MAXPEAK_Y_H              0x17
#define XL372_MAXPEAK_Y_L              0x18

#define XL372_MAXPEAK_Z_H              0x19
#define XL372_MAXPEAK_Z_L              0x1A

#define XL372_OFFSET_X                 0x20
#define XL372_OFFSET_Y                 0x21
#define XL372_OFFSET_Z                 0x22

#define XL372_THRESH_ACT_X_H           0x23
#define XL372_THRESH_ACT_X_L           0x24

#define XL372_THRESH_ACT_Y_H           0x25
#define XL372_THRESH_ACT_Y_L           0x26

#define XL372_THRESH_ACT_Z_H           0x27
#define XL372_THRESH_ACT_Z_L           0x28

#define XL372_TIME_ACT                 0x29

#define XL372_THRESH_INACT_X_H         0x2A
#define XL372_THRESH_INACT_X_L         0x2B

#define XL372_THRESH_INACT_Y_H         0x2C
#define XL372_THRESH_INACT_Y_L         0x2D

#define XL372_THRESH_INACT_Z_H         0x2E
#define XL372_THRESH_INACT_Z_L         0x2F

#define XL372_TIME_INACT_H             0x30
#define XL372_TIME_INACT_L             0x31

#define XL372_THRESH_ACT2_X_H          0x32
#define XL372_THRESH_ACT2_X_L          0x33

#define XL372_THRESH_ACT2_Y_H          0x34
#define XL372_THRESH_ACT2_Y_L          0x35

#define XL372_THRESH_ACT2_Z_H          0x36
#define XL372_THRESH_ACT2_Z_L          0x37

#define XL372_HPF                      0x38

#define XL372_FIFO_SAMPLES             0x39
#define XL372_FIFO_CTL                 0x3A

#define XL372_INT1_MAP                 0x3B
#define XL372_INT2_MAP                 0x3C

#define XL372_TIMING                   0x3D
#define XL372_MEASURE                  0x3E
#define XL372_POWER_CTL                0x3F
#define XL372_SELF_TEST                0x40
#define XL372_RESET                    0x41

#define XL372_FIFO_DATA                0x42


/*----------------------------------------------------------------------
  Bit field definitions and register values
  ----------------------------------------------------------------------*/
/* register values for Analog Devices ID
 * The Analog Devices ID should always read this value, 
 * The customer does not need to use this value but it can be used to 
 * check that the device can communicate */

#define	ADI_ID                         0xad

/* register values for Analog Devices MEMS ID
 * The Analog Devices MEMS ID should always read this value, 
 * The customer does not need to use this value but it can be used to 
 * check that the device can communicate */

#define	MEMS_ID                        0x1d

/* register values for part ID
 * The part ID should always read this value, 
 * The customer does not need to use this value but it can be used to 
 * check that the device can communicate */

#define	PART_ID                        0xfa

/* register values for revision ID
 * This register contains a product revision ID, beginning with 
 * 0x00 and is changed for each subsequent revision */

#define	REV_ID                         0x02

 
/*************************************************************************************************
* This portion purloined from
* https://github.com/analogdevicesinc/arduino/blob/master/Arduino%20Uno%20R3/examples/ADXL372_example/adxl372.h#L104
* and
* https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/drivers/iio/accel/adxl372.c?id=HEAD#n1
*************************************************************************************************/  

#define MEASURE_AUTOSLEEP_MASK		0xBF
#define MEASURE_BANDWIDTH_MASK		0xF8
#define MEASURE_ACTPROC_MASK		0xCF
#define TIMING_ODR_MASK			0x1F
#define TIMING_WUR_MASK 		0xE3
#define PWRCTRL_OPMODE_MASK		0xFC
#define PWRCTRL_INSTON_THRESH_MASK	0xDF
#define PWRCTRL_INSTON_THRESH_MASK	0xDF
#define PWRCTRL_FILTER_SETTLE_MASK	0xEF

#define MEASURE_AUTOSLEEP_POS		6
#define MEASURE_ACTPROC_POS		4
#define TIMING_ODR_POS			5
#define TIMING_WUR_POS 			2
#define INSTAON_THRESH_POS		5
#define FIFO_CRL_SAMP8_POS		0
#define FIFO_CRL_MODE_POS		1
#define FIFO_CRL_FORMAT_POS		3
#define PWRCTRL_FILTER_SETTLE_POS	4

#define DATA_RDY	1
#define FIFO_RDY	2
#define FIFO_FULL	4
#define FIFO_OVR	8

#define ADXL_SPI_RNW    1

/*Acceleremoter configuration*/
#define ACT_VALUE          30     /* Activity threshold value */

#define INACT_VALUE        30     /* Inactivity threshold value */

#define ACT_TIMER          1    /* Activity timer value in multiples of 3.3ms */

#define INACT_TIMER        1     /* Inactivity timer value in multiples of 26ms */

#define ADXL_INT1_PIN     7
#define ADXL_INT2_PIN     5

typedef unsigned char adxl_spi_handle;

typedef enum {
    STAND_BY = 0,
    WAKE_UP,
    INSTANT_ON,
    FULL_BW_MEASUREMENT
} ADXL372_OP_MODE;

typedef enum {
    ODR_400Hz = 0,
    ODR_800Hz,
    ODR_1600Hz,
    ODR_3200Hz,
    ODR_6400Hz
} ADXL372_ODR;

typedef enum {
    BW_200Hz = 0,
    BW_400Hz,
    BW_800Hz,
    BW_1600Hz,
    BW_3200Hz
} ADXL372_BW;

typedef enum {
    WUR_52ms = 0,
    WUR_104ms,
    WUR_208ms,
    WUR_512ms,
    WUR_2048ms,
    WUR_4096ms,
    WUR_8192ms,
    WUR_24576ms
} ADXL372_WUR;


typedef enum {
    DEF = 0,
    LINKED,
    LOOPED
} ADXL372_ACT_PROC_MODE;


typedef enum {
    XYZ_FIFO = 0,
    X_FIFO,
    Y_FIFO,
    XY_FIFO,
    Z_FIFO,
    XZ_FIFO,
    YZ_FIFO,
    XYZ_PEAK_FIFO
} ADXL372_FIFO_FORMAT;

typedef enum {
    BYPASSED = 0,
    STREAMED,
    TRIGGERED,
    OLDEST_SAVED
} ADXL372_FIFO_MODE;

typedef struct {
    unsigned short samples;
    ADXL372_FIFO_MODE mode;
    ADXL372_FIFO_FORMAT format;
} fifo_config_t;

typedef enum {
    ADXL_INSTAON_LOW_THRESH = 0,
    ADXL_INSTAON_HIGH_THRESH
} ADXL_INSTAON_THRESH;

typedef enum {
    FILTER_SETTLE_16 = 0,
    FILTER_SETTLE_370
} ADXL372_Filter_Settle;


typedef struct {
    short x;
    short y;
    short z;
} AccelTriplet_t; 


struct adxl372_device {
    adxl_spi_handle *spi;
    fifo_config_t fifo_config;
};
#endif /* __XL372_H */
